@@ -1,4 +1,7 @@
-#!/usr/bin/env python2
+# -*- coding: utf-8 -*-
+# docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
+# Please refer to the git history regarding who changed what and when in this file.
+
 '''A reliable mail-retriever toolkit.
 
 getmail is a reliable, modular, extensible mail retriever with support for
@@ -12,15 +15,19 @@ You should have received a copy of the license in the file COPYING.
 '''
 
 import sys
-if sys.hexversion < 0x2030300:
-    raise ImportError('getmail version 5 requires Python version 2.3.3'
-                      ' or later')
 
-__version__ = '5.16'
+__version__ = '6.18.12'
+__license__ = 'GNU GPL version 2'
+
+__py_required__ = '2.7.16'
+__py_required_hex__ = 0x20710f0
+
+if sys.hexversion < __py_required_hex__:
+    raise ImportError('getmail version %s requires Python version %s '
+                      'or later'%(__version__,__py_required__))
 
 __all__ = [
     'baseclasses',
-    'compatibility',
     'constants',
     'destinations',
     'exceptions',
