@@ -1,4 +1,7 @@
-#!/usr/bin/env python2.3
+# -*- coding: utf-8 -*-
+# docs/COPYING 2a + DRY: https://github.com/getmail6/getmail6
+# Please refer to the git history regarding who changed what and when in this file.
+
 '''Classes implementing destinations (files, directories, or programs getmail
 can deliver mail to).
 
@@ -12,23 +15,16 @@ Currently implemented:
     recipient address patterns)
 '''
 
-__all__ = [
-    'DeliverySkeleton',
-    'Maildir',
-    'Mboxrd',
-    'MDA_qmaillocal',
-    'MDA_external',
-    'MultiDestinationBase',
-    'MultiDestination',
-    'MultiSorterBase',
-    'MultiSorter',
-]
-
+import sys
 import os
 import re
-import tempfile
+import smtplib
 import types
-import email.Utils
+
+try:
+    import email.Utils as Utils
+except ImportError:
+    import email.utils as Utils
 
 import pwd
 
@@ -36,6 +32,25 @@ from getmailcore.exceptions import *
 from getmailcore.utilities import *
 from getmailcore.baseclasses import *
 
+__all__ = [
+    'DeliverySkeleton',
+    'Maildir',
+    'Mboxrd',
+    'MDA_qmaillocal',
+    'MDA_external',
+    'MDA_lmtp',
+    'MultiDestinationBase',
+    'MultiDestination',
+    'MultiSorterBase',
+    'MultiSorter',
+]
+
+def exit_with(stderr,errmsg):
+    stderr.write(errmsg.encode())
+    stderr.flush()
+    os.fsync(stderr.fileno())
+    os._exit(127)
+
 #######################################
 class DeliverySkeleton(ConfigurableBase):
     '''Base class for implementing message-delivery classes.
@@ -74,7 +89,7 @@ class DeliverySkeleton(ConfigurableBase):
         ConfigurableBase.__init__(self, **args)
         try:
             self.initialize()
-        except KeyError, o:
+        except KeyError as o:
             raise getmailConfigurationError(
                 'missing required configuration parameter %s' % o
             )
@@ -98,6 +113,14 @@ class DeliverySkeleton(ConfigurableBase):
         msg.received_by = self.received_by
         return self._deliver_message(msg, delivered_to, received)
 
+    def some_security(self):
+        if ((os.geteuid() == 0 or os.getegid() == 0)
+                and not self.conf['allow_root_commands']):
+            raise getmailConfigurationError(
+                'refuse to invoke external commands as root '
+                'or GID 0 by default'
+            )
+
 #######################################
 class Maildir(DeliverySkeleton, ForkingBase):
     '''Maildir destination.
@@ -120,7 +143,7 @@ class Maildir(DeliverySkeleton, ForkingBase):
         self.dcount = 0
         try:
             self.conf['filemode'] = int(self.conf['filemode'], 8)
-        except ValueError, o:
+        except ValueError as o:
             raise getmailConfigurationError('filemode %s not valid: %s'
                                             % (self.conf['filemode'], o))
 
@@ -151,17 +174,12 @@ class Maildir(DeliverySkeleton, ForkingBase):
                 self.conf['path'], msg.flatten(delivered_to, received),
                 self.hostname, self.dcount, self.conf['filemode']
             )
-            stdout.write(f)
+            stdout.write(f.encode())
             stdout.flush()
             os.fsync(stdout.fileno())
             os._exit(0)
-        except StandardError, o:
-            # Child process; any error must cause us to exit nonzero for parent
-            # to detect it
-            stderr.write('maildir delivery process failed (%s)' % o)
-            stderr.flush()
-            os.fsync(stderr.fileno())
-            os._exit(127)
+        except Exception as o:
+            exit_with(stderr, 'maildir delivery process failed (%s)' % o)
 
     def _deliver_message(self, msg, delivered_to, received):
         self.log.trace()
@@ -181,34 +199,22 @@ class Maildir(DeliverySkeleton, ForkingBase):
                     raise getmailConfigurationError(
                         'refuse to deliver mail as GID 0'
                     )
-        self._prepare_child()
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
 
-        if not childpid:
-            # Child
-            self.__deliver_message_maildir(uid, gid, msg, delivered_to,
-                                           received, stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
-
-        # Parent
-        exitcode = self._wait_for_child(childpid)
-
-        stdout.seek(0)
-        stderr.seek(0)
-        out = stdout.read().strip()
-        err = stderr.read().strip()
+        try:
+            child = self.forkchild(
+                lambda o,e: self.__deliver_message_maildir(
+                    uid, gid, msg, delivered_to, received, o, e)
+                )
+        except getmailOperationError as oe:
+            raise getmailDeliveryError( 'delivery %s child operation error: %s' % (self, oe) )
 
         self.log.debug('maildir delivery process %d exited %d\n'
-                       % (childpid, exitcode))
-
-        if exitcode or err:
+                       % (child.childpid, child.exitcode))
+        if child.exitcode or child.err:
             raise getmailDeliveryError('maildir delivery %d error (%d, %s)'
-                                       % (childpid, exitcode, err))
-
+                                       % (child.childpid, child.exitcode, child.err))
         self.dcount += 1
-        self.log.debug('maildir file %s' % out)
+        self.log.debug('maildir file %s' % child.out)
         return self
 
 #######################################
@@ -272,13 +278,16 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
             # Open mbox file, refusing to create it if it doesn't exist
             fd = os.open(self.conf['path'], os.O_RDWR)
             status_old = os.fstat(fd)
-            f = os.fdopen(fd, 'r+b')
+            try:
+                f = os.fdopen(fd, 'br+')
+            except ValueError: # py2
+                f = os.fdopen(fd, 'r+')
             lock_file(f, self.conf['locktype'])
             # Check if it _is_ an mbox file.  mbox files must start with "From "
             # in their first line, or are 0-length files.
             f.seek(0, 0)
             first_line = f.readline()
-            if first_line and not first_line.startswith('From '):
+            if first_line and not first_line.startswith(b'From '):
                 # Not an mbox file; abort here
                 unlock_file(f, self.conf['locktype'])
                 raise getmailDeliveryError('not an mboxrd file (%s)'
@@ -288,7 +297,7 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
             try:
                 # Write out message plus blank line with native EOL
                 f.write(msg.flatten(delivered_to, received, include_from=True,
-                                    mangle_from=True) + os.linesep)
+                                    mangle_from=True) + os.linesep.encode())
                 f.flush()
                 os.fsync(fd)
                 status_new = os.fstat(fd)
@@ -296,17 +305,17 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
                 try:
                     os.utime(self.conf['path'], (status_old.st_atime,
                              status_new.st_mtime))
-                except OSError, o:
+                except OSError as o:
                     # Not root or owner; readers will not be able to reliably
                     # detect new mail.  But you shouldn't be delivering to
                     # other peoples' mboxes unless you're root, anyways.
-                    stdout.write('failed to updated mtime/atime of mbox')
+                    stdout.write(b'failed to update mtime/atime of mbox')
                     stdout.flush()
                     os.fsync(stdout.fileno())
 
                 unlock_file(f, self.conf['locktype'])
 
-            except IOError, o:
+            except IOError as o:
                 try:
                     if not f.closed:
                         # If the file was opened and we know how long it was,
@@ -316,7 +325,7 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
                         f.truncate(status_old.st_size)
                 except KeyboardInterrupt:
                     raise
-                except StandardError:
+                except Exception:
                     pass
                 raise getmailDeliveryError(
                     'failure writing message to mbox file "%s" (%s)'
@@ -325,13 +334,8 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
 
             os._exit(0)
 
-        except StandardError, o:
-            # Child process; any error must cause us to exit nonzero for parent
-            # to detect it
-            stderr.write('mbox delivery process failed (%s)' % o)
-            stderr.flush()
-            os.fsync(stderr.fileno())
-            os._exit(127)
+        except Exception as o:
+            exit_with(stderr, 'mbox delivery process failed (%s)' % o)
 
     def _deliver_message(self, msg, delivered_to, received):
         self.log.trace()
@@ -353,37 +357,27 @@ class Mboxrd(DeliverySkeleton, ForkingBase):
                 raise getmailConfigurationError(
                     'refuse to deliver mail as GID 0'
                 )
-        self._prepare_child()
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
 
-        if not childpid:
-            # Child
-            self.__deliver_message_mbox(uid, gid, msg, delivered_to, received,
-                                        stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
-
-        # Parent
-        exitcode = self._wait_for_child(childpid)
-
-        stdout.seek(0)
-        stderr.seek(0)
-        out = stdout.read().strip()
-        err = stderr.read().strip()
+        try:
+            child = self.forkchild(
+                lambda o,e: self.__deliver_message_mbox(
+                    uid, gid, msg, delivered_to, received, o, e)
+                )
+        except getmailOperationError as oe:
+            raise getmailDeliveryError( 'delivery %s child operation error: %s' % (self, oe) )
 
         self.log.debug('mboxrd delivery process %d exited %d\n'
-                       % (childpid, exitcode))
-
-        if exitcode or err:
+                       % (child.childpid, child.exitcode))
+        if child.exitcode or child.err:
             raise getmailDeliveryError('mboxrd delivery %d error (%d, %s)'
-                                       % (childpid, exitcode, err))
+                                       % (child.childpid, child.exitcode, child.err))
 
-        if out:
-            self.log.debug('mbox delivery: %s' % out)
+        if child.out:
+            self.log.debug('mbox delivery: %s' % child.out)
 
         return self
 
+
 #######################################
 class MDA_qmaillocal(DeliverySkeleton, ForkingBase):
     '''qmail-local MDA destination.
@@ -474,7 +468,7 @@ class MDA_qmaillocal(DeliverySkeleton, ForkingBase):
     def showconf(self):
         self.log.info('MDA_qmaillocal(%s)\n' % self._confstring())
 
-    def _deliver_qmaillocal(self, msg, msginfo, delivered_to, received, stdout,
+    def __deliver_qmaillocal(self, msg, msginfo, delivered_to, received, stdout,
             stderr):
         try:
             args = (
@@ -484,53 +478,30 @@ class MDA_qmaillocal(DeliverySkeleton, ForkingBase):
                 self.conf['localdomain'], msginfo['sender'],
                 self.conf['defaultdelivery']
             )
-            self.log.debug('about to execl() with args %s\n' % str(args))
             # Modify message
             if self.conf['strip_delivered_to']:
                 msg.remove_header('delivered-to')
                 # Also don't insert a Delivered-To: header.
                 delivered_to = None
-            # Write out message
-            msgfile = tempfile.TemporaryFile()
-            msgfile.write(msg.flatten(delivered_to, received))
-            msgfile.flush()
-            os.fsync(msgfile.fileno())
-            # Rewind
-            msgfile.seek(0)
-            # Set stdin to read from this file
-            os.dup2(msgfile.fileno(), 0)
-            # Set stdout and stderr to write to files
-            os.dup2(stdout.fileno(), 1)
-            os.dup2(stderr.fileno(), 2)
-            change_usergroup(self.log, self.conf['user'], self.conf['group'])
-            # At least some security...
-            if ((os.geteuid() == 0 or os.getegid() == 0)
-                    and not self.conf['allow_root_commands']):
-                raise getmailConfigurationError(
-                    'refuse to invoke external commands as root '
-                    'or GID 0 by default'
-                )
 
-            os.execl(*args)
-        except StandardError, o:
-            # Child process; any error must cause us to exit nonzero for parent
-            # to detect it
-            stderr.write('exec of qmail-local failed (%s)' % o)
-            stderr.flush()
-            os.fsync(stderr.fileno())
-            os._exit(127)
+            change_usergroup(self.log, self.conf['user'], self.conf['group'])
+            self.some_security()
+            self.child_replace_me(msg,
+                            delivered_to, received, False,
+                            stdout, stderr, args)
+        except Exception as o:
+            exit_with(stderr,'exec of qmail-local failed (%s)' % o)
 
     def _deliver_message(self, msg, delivered_to, received):
         self.log.trace()
-        self._prepare_child()
         if msg.recipient == None:
             raise getmailConfigurationError(
                 'MDA_qmaillocal destination requires a message source that '
                 'preserves the message envelope'
             )
         msginfo = {
-            'sender' : msg.sender,
-            'local' : '@'.join(msg.recipient.lower().split('@')[:-1])
+            'sender' : msg.sender.strip(),
+            'local' : '@'.join(msg.recipient.strip().lower().split('@')[:-1])
         }
 
         self.log.debug('recipient: extracted local-part "%s"\n'
@@ -557,37 +528,28 @@ class MDA_qmaillocal(DeliverySkeleton, ForkingBase):
         self.log.debug('recipient: set dash to "%s", ext to "%s"\n'
                        % (msginfo['dash'], msginfo['ext']))
 
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
+        try:
+            child = self.forkchild(
+                lambda o,e: self.__deliver_qmaillocal(
+                    msg, msginfo, delivered_to, received, o, e)
+                )
+        except getmailOperationError as oe:
+            raise getmailDeliveryError( 'delivery %s child operation error: %s' % (self, oe) )
 
-        if not childpid:
-            # Child
-            self._deliver_qmaillocal(msg, msginfo, delivered_to, received,
-                                     stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
 
-        # Parent
-        exitcode = self._wait_for_child(childpid)
+        self.log.debug('qmail-local %d exited %d\n' % (child.childpid, child.exitcode))
 
-        stdout.seek(0)
-        stderr.seek(0)
-        out = stdout.read().strip()
-        err = stderr.read().strip()
-
-        self.log.debug('qmail-local %d exited %d\n' % (childpid, exitcode))
-
-        if exitcode == 111:
+        if child.exitcode == 111:
             raise getmailDeliveryError('qmail-local %d temporary error (%s)'
-                                       % (childpid, err))
-        elif exitcode:
+                                       % (child.childpid, child.err))
+        elif child.exitcode:
             raise getmailDeliveryError('qmail-local %d error (%d, %s)'
-                                       % (childpid, exitcode, err))
+                                       % (child.childpid, child.exitcode, child.err))
 
-        if out and err:
-            info = '%s:%s' % (out, err)
+        if child.out and child.err:
+            info = '%s:%s' % (child.out, child.err)
         else:
-            info = out or err
+            info = child.out or child.err
 
         return 'MDA_qmaillocal (%s)' % info
 
@@ -610,7 +572,7 @@ class MDA_external(DeliverySkeleton, ForkingBase):
                     %(recipient) - recipient address
                     %(domain) - domain-part of recipient address
                     %(local) - local-part of recipient address
-                    %(mailbox) - for IMAP retrievers, the name of the 
+                    %(mailbox) - for IMAP retrievers, the name of the
                         server-side mailbox/folder the message was retrieved
                         from.  Will be empty for POP.
 
@@ -639,6 +601,9 @@ class MDA_external(DeliverySkeleton, ForkingBase):
 
       ignore_stderr (boolean, optional) - if set, getmail will not consider the
             program writing to stderr to be an error.  The default is False.
+            
+      pipe_stdout (boolean, optional) - if set, stdout from external command
+            will be forwarded to stdout of getmail.  The default is True. 
     '''
     _confitems = (
         ConfInstance(name='configparser', required=False),
@@ -649,6 +614,7 @@ class MDA_external(DeliverySkeleton, ForkingBase):
         ConfBool(name='allow_root_commands', required=False, default=False),
         ConfBool(name='unixfrom', required=False, default=False),
         ConfBool(name='ignore_stderr', required=False, default=False),
+        ConfBool(name='pipe_stdout', required=False, default=True),
     )
 
     def initialize(self):
@@ -671,97 +637,141 @@ class MDA_external(DeliverySkeleton, ForkingBase):
     def showconf(self):
         self.log.info('MDA_external(%s)\n' % self._confstring())
 
-    def _deliver_command(self, msg, msginfo, delivered_to, received,
+    def __deliver_command(self, msg, msginfo, delivered_to, received,
                          stdout, stderr):
         try:
-            # Write out message with native EOL convention
-            msgfile = tempfile.TemporaryFile()
-            msgfile.write(msg.flatten(delivered_to, received,
-                                      include_from=self.conf['unixfrom']))
-            msgfile.flush()
-            os.fsync(msgfile.fileno())
-            # Rewind
-            msgfile.seek(0)
-            # Set stdin to read from this file
-            os.dup2(msgfile.fileno(), 0)
-            # Set stdout and stderr to write to files
-            os.dup2(stdout.fileno(), 1)
-            os.dup2(stderr.fileno(), 2)
-            change_usergroup(self.log, self.conf['user'], self.conf['group'])
-            # At least some security...
-            if ((os.geteuid() == 0 or os.getegid() == 0)
-                    and not self.conf['allow_root_commands']):
-                raise getmailConfigurationError(
-                    'refuse to invoke external commands as root '
-                    'or GID 0 by default'
-                )
             args = [self.conf['path'], self.conf['path']]
-            msginfo['mailbox'] = (self.retriever.mailbox_selected 
-                                  or '').encode('utf-8')
+            msginfo['mailbox'] = self.retriever.mailbox_selected or ''
+            if sys.version_info.major == 2:
+                msginfo['mailbox'] = msginfo['mailbox'].encode('utf-8')
             for arg in self.conf['arguments']:
                 arg = expand_user_vars(arg)
                 for (key, value) in msginfo.items():
                     arg = arg.replace('%%(%s)' % key, value)
                 args.append(arg)
-            self.log.debug('about to execl() with args %s\n' % str(args))
-            os.execl(*args)
-        except StandardError, o:
-            # Child process; any error must cause us to exit nonzero for parent
-            # to detect it
-            stderr.write('exec of command %s failed (%s)'
+
+            change_usergroup(self.log, self.conf['user'], self.conf['group'])
+            self.some_security()
+            self.child_replace_me(msg,
+                            delivered_to, received, self.conf['unixfrom'],
+                            stdout, stderr, args)
+        except Exception as o:
+            exit_with(stderr, 'exec of command %s failed (%s)'
                          % (self.conf['command'], o))
-            stderr.flush()
-            os.fsync(stderr.fileno())
-            os._exit(127)
 
     def _deliver_message(self, msg, delivered_to, received):
         self.log.trace()
-        self._prepare_child()
-        msginfo = {}
-        msginfo['sender'] = msg.sender
-        if msg.recipient != None:
-            msginfo['recipient'] = msg.recipient
-            msginfo['domain'] = msg.recipient.lower().split('@')[-1]
-            msginfo['local'] = '@'.join(msg.recipient.split('@')[:-1])
-        self.log.debug('msginfo "%s"\n' % msginfo)
-
-        stdout = tempfile.TemporaryFile()
-        stderr = tempfile.TemporaryFile()
-        childpid = os.fork()
-
-        if not childpid:
-            # Child
-            self._deliver_command(msg, msginfo, delivered_to, received,
-                                  stdout, stderr)
-        self.log.debug('spawned child %d\n' % childpid)
-
-        # Parent
-        exitcode = self._wait_for_child(childpid)
-
-        stdout.seek(0)
-        stderr.seek(0)
-        out = stdout.read().strip()
-        err = stderr.read().strip()
+        msginfo = self.get_msginfo(msg)
 
+        try:
+            child = self.forkchild(
+                lambda o,e: self.__deliver_command(
+                    msg, msginfo, delivered_to, received, o, e)
+                )
+        except getmailOperationError as oe:
+            raise getmailDeliveryError( 'delivery %s child operation error: %s' % (self, oe) )
+        
+        if self.conf['pipe_stdout'] and child.out:
+            self.log.info('%s\n' % (''.join(map(chr, child.out))))
+        
         self.log.debug('command %s %d exited %d\n'
-                       % (self.conf['command'], childpid, exitcode))
-
-        if exitcode:
+                       % (self.conf['command'], child.childpid, child.exitcode))
+        if child.exitcode:
             raise getmailDeliveryError(
                 'command %s %d error (%d, %s)'
-                % (self.conf['command'], childpid, exitcode, err)
+                % (self.conf['command'], child.childpid, child.exitcode, child.err)
             )
-        elif err:
+        elif child.err:
             if not self.conf['ignore_stderr']:
                 raise getmailDeliveryError(
                     'command %s %d wrote to stderr: %s'
-                    % (self.conf['command'], childpid, err)
+                    % (self.conf['command'], child.childpid, child.err)
                 )
             #else:
             # User said to ignore stderr, just log it.
-            self.log.info('command %s: %s' % (self, err))
+            self.log.info('command %s: %s' % (self, child.err))
+        return 'MDA_external command %s (%s)' % (self.conf['command'], child.out)
+
+
+class MDA_lmtp(DeliverySkeleton):
+    """LMTP server destination."""
+    _confitems = (
+        ConfInstance(name='configparser', required=False),
+        ConfString(name='host', required=True),
+        ConfInt(name='port', required=False, default=smtplib.LMTP_PORT),
+        ConfString(name='fallback', required=False, default=None),
+        ConfString(name='override', required=False, default=None),
+    )
+
+    def initialize(self):
+        if sys.version_info.major < 3:
+            raise getmailConfigurationError(
+                'MDA_lmtp in config: that was implemented only for python 3'
+            )
+        self.log.trace()
+        self.__connect()
+
+    def __connect(self):
+        try:
+            self.server = smtplib.LMTP(self.conf['host'], self.conf['port'])
+        except smtplib.SMTPException as err:
+            raise getmailConfigurationError(
+                'Failed to connect to server'
+            )
+
+    def __str__(self):
+        self.log.trace()
+        host = self.conf['host']
+        port = self.conf['port']
+        if host.startswith('/') or not port:
+            return 'MDA_lmtp at %s' % host
+        else:
+            return 'MDA_lmtp at %s:%d' % (host, port)
+
+    def showconf(self):
+        self.log.info('%s(%s)' % (type(self).__name__, self._confstring()))
+
+    def __send(self, msg, sender, recipient, __retrying=False):
+        try:
+            rcpt = self.server.send_message(msg, sender, recipient)
+        except smtplib.SMTPServerDisconnected as err:
+            if __retrying:
+                raise
+            else:
+                self.log.info('Lost connection to LMTP server, reconnecting')
+                self.__connect()
+                return self.__send(msg, sender, recipient, __retrying=True)
+        except smtplib.SMTPRecipientsRefused as err:
+            rcpt = err.recipients
+        except smtplib.SMTPException as err:
+            raise getmailDeliveryError(
+                'LMTP error: %s: %s' % (type(err).__name__, err)
+            )
+
+        return rcpt
+
+    def _deliver_message(self, msg, delivered_to, received):
+        self.log.trace()
+        recipient = self.conf.get('override', msg.recipient)
+        rcpt = self.__send(msg.content(), msg.sender, recipient)
+
+        if recipient in rcpt:
+            status, error = rcpt[recipient]
+            fb_recipient = self.conf['fallback']
+            if 500 <= status <= 599 and fb_recipient:
+                fb_rcpt = self.__send(msg.content(), msg.sender, fb_recipient)
+                if fb_recipient in fb_rcpt:
+                    raise getmailDeliveryError(
+                        'Cannot deliver to intended or fallback target: %d %s'
+                        % fb_rcpt[fb_recipient]
+                    )
+            else:
+                raise getmailDeliveryError(
+                    'Cannot deliver to intended target: %d %s'
+                    % rcpt[recipient]
+                )
+        return str(self)
 
-        return 'MDA_external command %s (%s)' % (self.conf['command'], out)
 
 #######################################
 class MultiDestinationBase(DeliverySkeleton):
@@ -857,7 +867,7 @@ class MultiDestination(MultiDestinationBase):
         for item in dests:
             try:
                 dest = self._get_destination(item)
-            except getmailConfigurationError, o:
+            except getmailConfigurationError as o:
                 raise getmailConfigurationError('%s destination error %s'
                                                 % (item, o))
             self._destinations.append(dest)
@@ -914,13 +924,13 @@ class MultiSorterBase(MultiDestinationBase):
             for (pattern, path) in _locals:
                 try:
                     dest = self._get_destination(path)
-                except getmailConfigurationError, o:
+                except getmailConfigurationError as o:
                     raise getmailConfigurationError(
                         'pattern %s destination error %s' % (pattern, o)
                     )
                 self.targets.append((re.compile(pattern, re.IGNORECASE), dest))
                 self._destinations.append(dest)
-        except re.error, o:
+        except re.error as o:
             raise getmailConfigurationError('invalid regular expression %s' % o)
 
     def _confstring(self):
@@ -1056,7 +1066,7 @@ class MultiGuesser(MultiSorterBase):
                     'looking for addresses in %s header fields\n' % field
                 )
                 header_addrs.extend(
-                    [addr for (name, addr) in email.Utils.getaddresses(
+                    [addr for (name, addr) in Utils.getaddresses(
                         msg.get_all(field, [])
                      ) if addr]
                 )
