@@ -34,17 +34,17 @@ from getmailcore.baseclasses import *
 from getmailcore._retrieverbases import *
 
 __all__ = [
+    'SimplePOP3Retriever',
+    'SimplePOP3SSLRetriever',
     'BrokenUIDLPOP3Retriever',
     'BrokenUIDLPOP3SSLRetriever',
-    'MultidropIMAPRetriever',
-    'MultidropIMAPSSLRetriever',
     'MultidropPOP3Retriever',
     'MultidropPOP3SSLRetriever',
     'MultidropSDPSRetriever',
     'SimpleIMAPRetriever',
     'SimpleIMAPSSLRetriever',
-    'SimplePOP3Retriever',
-    'SimplePOP3SSLRetriever',
+    'MultidropIMAPRetriever',
+    'MultidropIMAPSSLRetriever',
 ]
 
 
@@ -66,7 +66,6 @@ class SimplePOP3Retriever(POP3RetrieverBase, POP3initMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
-        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
     )
@@ -101,7 +100,6 @@ class SimplePOP3SSLRetriever(POP3RetrieverBase, POP3SSLinitMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
-        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfBool(name='delete_dup_msgids', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
@@ -180,7 +178,6 @@ class BrokenUIDLPOP3Retriever(BrokenUIDLPOP3RetrieverBase, POP3initMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
-        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
     )
     received_with = 'POP3'
@@ -212,7 +209,6 @@ class BrokenUIDLPOP3SSLRetriever(BrokenUIDLPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
-        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
@@ -251,7 +247,6 @@ class MultidropPOP3Retriever(MultidropPOP3RetrieverBase, POP3initMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
-        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
     )
@@ -286,7 +281,6 @@ class MultidropPOP3SSLRetriever(MultidropPOP3RetrieverBase, POP3SSLinitMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
-        ConfBool(name='use_xoauth2', required=False, default=False),
         ConfBool(name='use_apop', required=False, default=False),
         ConfString(name='envelope_recipient'),
         ConfFile(name='keyfile', required=False, default=None),
@@ -335,7 +329,6 @@ class MultidropSDPSRetriever(SimplePOP3Retriever, POP3initMixIn):
         ConfString(name='username'),
         ConfPassword(name='password', required=False, default=None),
         ConfTupleOfStrings(name='password_command', required=False, default=()),
-        ConfBool(name='use_xoauth2', required=False, default=False),
         # Demon apparently doesn't support APOP
         ConfBool(name='use_apop', required=False, default=False),
     )
@@ -393,7 +386,6 @@ class SimpleIMAPRetriever(IMAPRetrieverBase, IMAPinitMixIn):
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
-        ConfString(name='uid_cache', required=False, default=None),
         ConfBool(name='record_mailbox', required=False, default=True),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
@@ -402,7 +394,6 @@ class SimpleIMAPRetriever(IMAPRetrieverBase, IMAPinitMixIn):
         ConfBool(name='use_xoauth2', required=False, default=False),
         ConfString(name='imap_search', required=False, default=None),
         ConfString(name='imap_on_delete', required=False, default=None),
-        ConfBool(name='imap_id_extension', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4'
@@ -429,6 +420,8 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInstance(name='configparser', required=False),
         ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
+        # socket.ssl() and socket timeouts were incompatible in Python 2.3;
+        # if you have problems, comment this line out
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
@@ -439,7 +432,6 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
-        ConfString(name='uid_cache', required=False, default=None),
         ConfBool(name='record_mailbox', required=False, default=True),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
@@ -455,7 +447,6 @@ class SimpleIMAPSSLRetriever(IMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfString(name='ssl_cert_hostname', required=False, default=None),
         ConfString(name='imap_search', required=False, default=None),
         ConfString(name='imap_on_delete', required=False, default=None),
-        ConfBool(name='imap_id_extension', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -492,7 +483,6 @@ class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
-        ConfString(name='uid_cache', required=False, default=None),
         ConfBool(name='record_mailbox', required=False, default=True),
         # imaplib.IMAP4.login_cram_md5() requires the (unimplemented)
         # .authenticate(), so we can't do this yet (?).
@@ -502,7 +492,6 @@ class MultidropIMAPRetriever(MultidropIMAPRetrieverBase, IMAPinitMixIn):
         ConfString(name='envelope_recipient'),
         ConfString(name='imap_search', required=False, default=None),
         ConfString(name='imap_on_delete', required=False, default=None),
-        ConfBool(name='imap_id_extension', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4'
@@ -529,6 +518,8 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfInstance(name='configparser', required=False),
         ConfDirectory(name='getmaildir', required=False, default='~/.getmail/'),
 
+        # socket.ssl() and socket timeouts were incompatible in Python 2.3;
+        # if you have problems, comment this line out
         ConfInt(name='timeout', required=False, default=180),
         ConfString(name='server'),
         ConfInt(name='port', required=False, default=imaplib.IMAP4_SSL_PORT),
@@ -539,7 +530,6 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
                            default="('INBOX', )", allow_specials=('ALL',)),
         ConfBool(name='use_peek', required=False, default=True),
         ConfString(name='move_on_delete', required=False, default=None),
-        ConfString(name='uid_cache', required=False, default=None),
         ConfBool(name='record_mailbox', required=False, default=True),
         ConfFile(name='keyfile', required=False, default=None),
         ConfFile(name='certfile', required=False, default=None),
@@ -556,7 +546,6 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         ConfString(name='ssl_cert_hostname', required=False, default=None),
         ConfString(name='imap_search', required=False, default=None),
         ConfString(name='imap_on_delete', required=False, default=None),
-        ConfBool(name='imap_id_extension', required=False, default=False),
     )
     received_from = None
     received_with = 'IMAP4-SSL'
@@ -574,3 +563,4 @@ class MultidropIMAPSSLRetriever(MultidropIMAPRetrieverBase, IMAPSSLinitMixIn):
         self.log.trace()
         self.log.info('MultidropIMAPSSLRetriever(%s)' % self._confstring()
                       + os.linesep)
+
